package io.gitlab.jfronny.translater.transformer;

import io.gitlab.jfronny.libjf.translate.api.TranslateException;
import io.gitlab.jfronny.libjf.translate.impl.google.GoogleTranslateService;
import io.gitlab.jfronny.libjf.translate.impl.google.Language;
import io.gitlab.jfronny.translater.Cfg;
import io.gitlab.jfronny.translater.Translater;
import org.apache.commons.lang3.StringUtils;

import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class TranslatingTransformer implements Transformer {
    private static final Pattern SURROUNDING_SPACE_PATTERN = Pattern.compile("^(\\s*)(.*[^\\s]+)(\\s*)$");
    private final Random rnd;
    private final Language[] languages = Language.values();
    private final GoogleTranslateService ts = new GoogleTranslateService();
    @Override
    public String transform(String str) {
        try {
            if (str.contains("%")) {
                StringBuilder res = new StringBuilder();
                boolean f = true;
                for (String s : str.split("%")) {
                    if (!f) {
                        res.append("%");
                        if (s.length() > 0)
                            res.append(s.charAt(0)).append(transform(s.substring(1)).replace("%", ""));
                    } else
                        res.append(transform(s));
                    f = false;
                }
                return res.toString();
            } else if (str.contains("$")) {
                StringBuilder res = new StringBuilder();
                boolean f = true;
                for (String s : str.split("\\$")) {
                    if (!f) {
                        res.append("$").append(s.charAt(0)).append(transform(s.substring(1)).replace("$", ""));
                    } else
                        res.append(transform(s));
                    f = false;
                }
                return res.toString();
            } else if (str.contains("§")) {
                StringBuilder res = new StringBuilder();
                boolean f = true;
                for (String s : str.split("§")) {
                    if (!f) {
                        res.append("§").append(s.charAt(0)).append(transform(s.substring(1)).replace("§", ""));
                    } else
                        res.append(transform(s));
                    f = false;
                }
                return res.toString();
            } else {
                return translateMultiple(str);
            }
        }
        catch (Exception e) {
            Translater.LOGGER.warn("Failed to transform: \"" + str + "\" (" + str.length() + " characters)");
            Translater.LOGGER.warn("Please report this bug with the mod containing the lang file");
            return null;
        }
    }

    private String translateMultiple(String str) throws TranslateException {
        Matcher m = SURROUNDING_SPACE_PATTERN.matcher(str);
        if (!m.find()) {
            Translater.LOGGER.info("Skipping translation of \"" + str + "\"");
            return str;
        }
        try {
            Language startLang = StringUtils.isAlpha(Cfg.targetLanguage) && Cfg.targetLanguage.length() == 2
                    ? Language.byId(Cfg.targetLanguage)
                    : Language.AUTO_DETECT; // Identify target language
            String currentState = m.group(2);
            Language currentLang = startLang;
            for (int i = 0; i < Cfg.rounds; i++) { // Translate around
                Language newLang = randomLanguage();
                currentState = ts.translate(currentState, Cfg.breakFully ? randomLanguage() : currentLang, newLang);
                currentLang = newLang;
            }
            currentState = ts.translate(currentState, currentLang, startLang == Language.AUTO_DETECT ? Language.ENGLISH : startLang); // Translate to starting language
            currentState = m.group(1) + currentState + m.group(3); // Add back surrounding white space
            Translater.LOGGER.info("Transformed: \"" + str + "\" to: \"" + currentState + "\"");
            return currentState;
        } catch (Exception e) {
            Translater.LOGGER.warn("Failed to break: \"" + m.group(2) + "\" (" + m.group(2).length() + " characters). Is your API key valid?");
            throw e;
        }
    }

    private Language randomLanguage() {
        return languages[rnd.nextInt(languages.length)];
    }

    public TranslatingTransformer() {
        rnd = new Random();
    }
}
